% calculate_thickness - Thickness calculation
%
% This script processes the data:
% 1- Compute TRT thickness (ILM-BM)
% 2- Locate the foveal center and align the images
% 3. Resample the data to another spatial pattern
%
% 2021, Mondragon Unibertsitatea, Biomedical Engineering Department
% -------------------------------------------------------------------------

clearvars;clc;close all;

% Add required functions
addpath(genpath('../utils'));

% ---------------------- Folder Structure --------------------------------
% Set Input path
rawPath = '../../data/raw';
inputPath = '../../data/derivatives/1_preprocessing';

% Create output folder if necessary. Delete if it was already created.
outputPath = '../../data/derivatives/2_thickness';

if ~exist(outputPath,'dir')
    mkdir(outputPath);
end
% ------------------------------------------------------------------------

% ------------------- Visual check of results --------------------------
savePng = true;
visible = 'off';

%-----------------------------------------------------------------------

% Load subjects list 
opts = detectImportOptions([rawPath '/subjects.tsv'],'FileType','text');
subjects = readtable([rawPath '/subjects.tsv'],opts);

nSubject = size(subjects,1);

% Choose one eye per subject
eyes = chooseEyes(subjects);

% --------------------- Processing characteristics -----------------------
% Interpolation method
interpMethod = 'cubic'; % linear, cubic

% Fovea location method
alignMethod = {'none','smooth_min','min','smooth_min'};

% Resample A-Scans to another pattern
Grid1.type = 'regular';
Grid1.maxD = 1.5;
Grid1.step = 0.02;
Grid1.nAngles = 0;

Grid2.type = 'star';
Grid2.maxD = 2.00;
Grid2.step = 0.02;
Grid2.nAngles = 24;

Grid = {Grid1, Grid2};

Combos = getCombos(alignMethod,Grid);
nCombos = length(Combos);

% -------------------------------------------------------------------

% ------------------------ Processing -------------------------------

for iCombo = 1:nCombos
    
    % Get parameters of each combo
    alignMethod = Combos(iCombo).alignMethod;
    Grid = Combos(iCombo).Grid;
    
    % Create folder of each combo
    comboPath = [outputPath '/' alignMethod '_' Grid.type '_' ... 
        num2str(Grid.maxD) '_' num2str(Grid.step) '_' num2str(Grid.nAngles)];
    
    if exist(comboPath,'dir')
        rmdir(comboPath,'s');
    end
    mkdir(comboPath);
    
    % Create folder to save pngs of each combo
    if savePng
        if ~exist([comboPath '/0_png'],'dir')
            mkdir([comboPath '/0_png']);
        end
    end

    scanTypeList = {'raster','star'};
    
    for iSubject = 1:nSubject
        
        for iType=1:2
            % Get scanType
            scanType = scanTypeList{iType};
            
            % Get scan characteristics
            subject = subjects.subject{iSubject};
            eye = eyes{iSubject};

            % load oct data
            searchPath = [inputPath '/' subject '/'  eye ];
            searchFile = [subject '_' eye '_' scanType '_preproc.mat'];
            [xAScan_0,yAScan_0,ILM_0,BM_0] = parload([searchPath '/' searchFile],...
            'xAScan','yAScan','ILM','BM');        
            
            % Compute thickness values derived from preprocessing 
            TRT_0 = ILM_0 - BM_0;
          
            % Foveal location and alignment
            [xAScan_1,yAScan_1,offset] = alignCentre(xAScan_0,yAScan_0,TRT_0,interpMethod,alignMethod);                           

            % Resample grid                    
            [xAScan,yAScan,TRT] = resampleAScan(xAScan_1,yAScan_1,TRT_0,interpMethod,Grid);

            % Define new folder and file
            newPath = [comboPath '/' subject '/' eye];
            newFile = [subject '_' eye '_' scanType '_thickness.mat'];

            % Create new folder and save file
            if ~exist(newPath,'dir')
                mkdir(newPath);                    
            end

            parsave([newPath '/' newFile],xAScan,yAScan,offset,TRT);
            
            if savePng
              % save for visual inspection
                f = figure('visible',visible);
                clf(f);

                subplot(131);hold on;
                surf(xAScan_0,yAScan_0,TRT_0,'EdgeColor','none');
                scatter3(0,0,max(TRT_0(:)),'r','filled');
                view(0,90);title('acquired');xlim([-2 2]);ylim([-2 2]);
            
                subplot(132);hold on;
                surf(xAScan_1,yAScan_1,TRT_0,'EdgeColor','none');
                scatter3(-offset(1),-offset(2),max(TRT_0(:)),'r','filled');                
                scatter3(0,0,max(TRT_0(:)),'g','filled');
                view(0,90);title('aligned');xlim([-2 2]);ylim([-2 2]);
                
                subplot(133);hold on;
                surf(xAScan,yAScan,TRT,'EdgeColor','none');
                scatter3(-offset(1),-offset(2),max(TRT(:)),'r','filled');                
                scatter3(0,0,max(TRT(:)),'g','filled');
                view(0,90);title('resampled');xlim([-2 2]);ylim([-2 2]);    
                     
                sgtitle(newFile,'Interpreter','none');

                saveas(f,[comboPath '/0_png/' newFile],'png');
                close(f);
            end
            %-----------------------------------------------------------------

            disp(newFile);
        end
    end             
end
