% export_results_d2 - Export foveal pit mathematical model comparison 
% results
%
% This script merges the results related to the foveal pit mathematical 
% model comparison:
% 1- Compute RMSE
% 2- Average paramter estimation for each eye
% 3- Compute estimation bias
% 4- Merge all results in a single table
%
% 2021, Mondragon Unibertsitatea, Biomedical Engineering Department
% -------------------------------------------------------------------------
%% Load data
close all;clc;clearvars;

addpath(genpath('../utils'));

dataPath = '../../data/derivatives/3_morphology';

% Load subjects list 
opts = detectImportOptions('../../data/raw/subjects.tsv','FileType','text');
subjects = readtable('../../data/raw/subjects.tsv',opts);
nSubject = size(subjects,1);

% Choose one eye per subject
eyes = chooseEyes(subjects);

Grid.type = 'star';
Grid.maxD = 2;
Grid.step = 0.02;
Grid.nAngles = 24;

% Pit Models
pitModelList = horzcat({'none','Scheibe','Ding','Yadav','Breher','Dubis','Liu'},...
    combineCell('movAvg_',5:5:60),...
    combineCell('loess_',1:50));

nPitModel = length(pitModelList);

% Morphological parameters
paramList = {'CFT','rimHeight','rimRadius','maxSlope'};

nParam = length(paramList);

% Scan types
scanTypeList = {'raster','star'};


for i = 1:nPitModel
    
    pitModel = pitModelList{i};    
       
    for iSubject = 1:nSubject
        
        % Get subjectId and eye
        subject = subjects.subject{iSubject};
        eye = eyes{iSubject};
                
        % ----------------- Modelled --------------------------
        % Get file path
        filePath = [dataPath '/' pitModel '/' subject '/' eye];
                
        % Get Modelled data from Raster
        scanType = 'raster';
        fileName = [subject '_' eye '_' scanType '_morphology.mat'];        
        load([filePath '/' fileName],'TRT_fit','Paramorf');
        
        TRT_fit_all.raster.(pitModel)(iSubject) = {TRT_fit.*1e3};
        Param.raster.(pitModel)(iSubject) = Paramorf;

        % Get Modelled data from Star
        scanType = 'star';
        fileName = [subject '_' eye '_' scanType '_morphology.mat'];      
        load([filePath '/' fileName],'TRT_fit','Paramorf');
        
        TRT_fit_all.star.(pitModel)(iSubject) = {TRT_fit.*1e3};        
        Param.star.(pitModel)(iSubject) = Paramorf;

        % -------------------------------------------------------        
    end
% i
end
clear Paramorf alignMethod modelPath morphPath Grid pitModel scanType
clear fileName eyePath i iEye eye subject interpNanMethod

% Create empty table to store the results
T = table;
T.subject = subjects.subject;

%% RMSE
for i = 1:nPitModel % don't get raw data
    
    pitModel = pitModelList{i};  
    
    % Save RMSE in raster scans
    varName = [pitModel '_raster_RMSE'];    
    T.(varName) = cell2mat(cellfun(@(x,y) sqrt(nanmean((x(:)-y(:)).^2)),...
            TRT_fit_all.raster.none,TRT_fit_all.raster.(pitModel),'UniformOutput',false))';         
    
    % Save RMSE in star scans
    varName = [pitModel '_star_RMSE'];    
    T.(varName) = cell2mat(cellfun(@(x,y) sqrt(nanmean((x(:)-y(:)).^2)),...
            TRT_fit_all.star.none,TRT_fit_all.star.(pitModel),'UniformOutput',false))';                     
end


%% Parameter estimation
for j=1:nParam
    param = paramList{j};
    
    for i=1:nPitModel
        pitModel = pitModelList{i};
        
        % Save each param in raster scans
        varName = [pitModel '_raster_' param];
        T.(varName) = mean(vertcat(Param.raster.(pitModel).(param)),2);
        
        % Save each param in star scans
        varName = [pitModel '_star_' param];
        T.(varName) = mean(vertcat(Param.star.(pitModel).(param)),2);                
    end
end


%% Bias
for j=1:nParam
    param = paramList{j};
    
    for i=1:nPitModel
        pitModel = pitModelList{i};
        
        % Bias in raster scans
        varName = [pitModel '_raster_' param '_bias'];
        xraw = T.(['none_raster_' param]);
        xmodel = T.([pitModel '_raster_' param]);
        
        T.(varName) = 100*(xmodel-xraw)./xraw;  
        
        % Bias in star scans
        varName = [pitModel '_star_' param '_bias'];
        xraw = T.(['none_star_' param]);
        xmodel = T.([pitModel '_star_' param]);
        
        T.(varName) = 100*(xmodel-xraw)./xraw;          
    end
end


%% Save table
writetable(T,'../data_analysis/results_2.csv');