function [X,Y,offset] = alignCentre(X,Y,TRT,interpMethod,alignMethod)
% alignCentre - Align the TRT thickness map by locating the foveal center
% 
% [X,Y,offset] = alignCentre(X,Y,TRT,interpMethod,alignMethod)
%
% Input arguments:
%   X: matrix of A-Scan x coordinates
%   Y: matrix of A-Scan y coordinates
%   TRT: matrix of TRT values corresponding to each coordinate
%   interpMethod: method used for interpolating in resample + min and 
% smooth + min
%   alignMethod: method used to locate the foveal center
%
% Output arguments:
%   X: matrix of A-Scan x coordinates after centering
%   Y: matrix of A-Scan y coordinates after centering
%   offset: 1x2 array with the distance of x and y axes between the
%   acquired and located foveal center points.
%
%
%  IMPORTANT: the smooth+min method is based on the foveafinder.m function
%  of AURA Tools. If you use it, please provide appropriate credit to the
%  original work (https://www.nitrc.org/projects/aura_tools/).
%
%  2021, Mondragon Unibertsitatea, Biomedical Engineering Department


switch alignMethod
    case 'none'     % default (without any alignemnt)
        Xmin = 0;
        Ymin = 0;
    
    case 'min'
        % Search only in the central region
        maxR = 0.85; 
        R = sqrt(X.^2+Y.^2);
        maskCentral = R <= maxR;
        TRT_aux = TRT;
        TRT_aux(~maskCentral) = nan;
        
        % get min
        [~,indmin] = min(TRT_aux(:));
        [indx,indy] = ind2sub(size(X),indmin);        
        Xmin = X(indx,indy);
        Ymin = Y(indx,indy);        
     case 'resample_min'
        % Interpolate to small regular grid 
        maxD = 0.85; 
        step = 0.02; 
        [Xg,Yg] =meshgrid(-maxD:step:maxD,-maxD:step:maxD);        
                
        TRT_grid = reshape(griddata(X(:),Y(:),TRT(:),Xg(:),Yg(:),...
            interpMethod),size(Xg));
  
        [~,indmin] = min(TRT_grid(:));
        [indx, indy] = ind2sub(size(TRT_grid),indmin);
        Xmin = Xg(indx,indy);
        Ymin = Yg(indx,indy);   
    case 'smooth_min'
        % Based on AURA foveaFinder.m function
        
        % Interpolate to small regular grid 
        maxD = 0.85; % 
        step = 0.02; 
        [Xg,Yg] =meshgrid(-maxD:step:maxD,-maxD:step:maxD);        
                
        TRT_grid = reshape(griddata(X(:),Y(:),TRT(:),Xg(:),Yg(:),...
            interpMethod),size(Xg));
        
        % Minimum thickness average in 0.05 mm radius circle
        maxR = 0.05;
        R = Xg.^2+Yg.^2;
        m = R < maxR;
        TRT_filt = imfilter(TRT_grid,double(m)/sum(m(:)),'replicate');

        [~,indmin] = min(TRT_filt(:));
        [indx, indy] = ind2sub(size(TRT_filt),indmin);
        Xmin = Xg(indx,indy);
        Ymin = Yg(indx,indy);

    case 'otherwise'
        error('Unsupported alignment method');
end

% Correct coordinates (2D traslation)
X = X - Xmin;
Y = Y - Ymin;

offset = [Xmin Ymin];
