function [TRT_fit,fitCoeff] = modelDubis(rho,TRT)
% modelDubis - fit the foveal pit model presented in Dubis et al. ,2009 
%
% [TRT_fit,fitCoeff] = modelDubis(rho,TRT)
%
% Input arguments:
%   rho: matrix with rho coordinates
%   TRT: matrix with TRT points (each row is an angular direction)
%
% Output arguments:
%   TRT_fit: fitted TRT values
%   fitCoeff: estimated model coefficients
%
%  2021, Mondragon Unibertsitatea, Biomedical Engineering Department


[nAngles, nPoints] = size(TRT);

TRT_fit = nan(nAngles,nPoints);

nBScans = nAngles/2;

fitCoeff = nan(nBScans,7);

%  Fitting configuration
x0 = [  0.2 0.14 0    0   5  0.09 0.8];
lower = [0   0  -0.3 -0.3 0 0 0];
upper = [1 0.3 0.3  0.1 30 10  2];

fitfun = fittype(@(a1,a2,b1,b2,c1,c2,z,x) ...
    a1.*exp((x-b1).^2./(-2*c1)) - a2.*exp((x-b2).^2./(-2*c2)) + z);
options = fitoptions('StartPoint',x0,'Method','NonlinearLeastSquares',...
    'Lower',lower,'Upper',upper,'TolFun',1e-6,'TolX',1e-6,'MaxIter',1000,'Display','off');

for n=1:nBScans
    
    % Reconstruct the B-Scan
    x = [-fliplr(rho(n+nBScans,:)) rho(n, 2:end)];
    y = [fliplr(TRT(n+nBScans,:)) TRT(n, 2:end)]; % Dont get center two times
    
    % Fit the model
    fitted = fit(x',y',fitfun,options);
    
    % Reconstruct fitted curve
    yFit = fitted(x)';
        
    % From fitted B-Scan to radial again
    TRT_fit(n,:) = yFit((end-nPoints+1):end);% Right part of the B-Scan
    TRT_fit(n+nBScans,:) = yFit(nPoints:-1:1);
    
    % Store coefficients
    fitCoeff(n,:) = [fitted.a1 fitted.a2 fitted.b1 fitted.b2 fitted.c1 fitted.c2 fitted.z];
end
